@@ -31,25 +31,25 @@ class Resolver():
         paths = os.environ.get('PATH', None)
         if paths is None:
             LOGGER.info('PATH environment variable does not exist!')
-            return None
-        # Iterate in reverse so we resolve tools in local paths after system paths
-        for path in paths.split(':')[::-1]:
-            # Try and remove the file from the path to include the parent dir
-            if os.path.isfile(path):
-                path = '/'.join(path.split('/')[:-1])
+        else:
+            # Iterate in reverse so we resolve tools in local paths after system paths
+            for path in paths.split(':')[::-1]:
+                # Try and remove the file from the path to include the parent dir
                 if os.path.isfile(path):
+                    path = '/'.join(path.split('/')[:-1])
+                    if os.path.isfile(path):
+                        print(
+                            f'{path} listed in PATH environment variable, refers to file!')
+                        continue
+                try:
+                    programs = os.listdir(self.resolve_home_dir(path))
+                except FileNotFoundError:
                     print(
-                        f'{path} listed in PATH environment variable, refers to file!')
+                        f'{path} listed in PATH environment variable, but does not exist!')
                     continue
-            try:
-                programs = os.listdir(self.resolve_home_dir(path))
-            except FileNotFoundError:
-                print(
-                    f'{path} listed in PATH environment variable, but does not exist!')
-                continue
-            self._paths.update(
-                {programs: f'{path}/{programs}' for programs in programs}
-            )
+                self._paths.update(
+                    {programs: f'{path}/{programs}' for programs in programs}
+                )
 
     def get(self, program: str) -> str:
         """
